<?php

namespace Teknisa\CRM\Factory;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class AgrupamentoParceiroFactory
{

    protected $connection;

    public function __construct(EntityManager $entityManager)
    {

        $this->connection = $entityManager->getConnection();
    }

    public function queryDeleteGroup($paramsQuery)
    {
        try {
            $delete = "DELETE FROM CRM_AGRUPAMENTOPARCEIRO
                        WHERE NRAGRPARC = :NRAGRPARC
                        AND CRM_NRORGTRAB = :CRM_NRORGTRAB
                        AND NRORG = :NRORG";

            $std = $this->connection->prepare($delete);
            $std->execute($paramsQuery);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verifyGroup($paramsQuery)
    {
        try {
            $query = "SELECT 
                         NRAGRPARC, 
                         NMPRINCIPALPARC,
                         NRINSCRICAOPARC
                       FROM 
                         CRM_AGRUPAMENTOPARCEIRO AGR
                         LEFT JOIN PARCNEGOCIO PARC
                           ON PARC.NRPARCNEGOCIO = AGR.NRPARCNEGOCIO
                        WHERE 
                          AGR.NRPARCNEGOCIO_REL = :NRPARCNEGOCIO_REL
                          AND AGR.CRM_NRORGTRAB = :CRM_NRORGTRAB
                          AND AGR.NRORG = :NRORG";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);

            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp[0];
            } else {
                return $resp;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insertTaskRel($paramsQuery)
    {
        try {
            $query = "INSERT INTO FILACONTATO (NRSEQFILACONTA, NRSEQCONTATO, DSFILACONTATO, DTPLANFILACONTA, CDOPERREAL,
                NRPARCNEGOCIO, IDORIGFILACONTA,NRPRIORFILACONT, NRORG, CRM_NRORGTRAB,NRSEQCAMPANHA)
            VALUES (:NRSEQFILACONTA,:NRSEQCONTATO,:DSFILACONTATO,:DTPLANFILACONTA,:CDOPERREAL,
                 :NRPARCNEGOCIO,:IDORIGFILACONTA,:NRPRIORFILACONT, :NRORG, :CRM_NRORGTRAB,:NRSEQCAMPANHA)";
            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
        } catch (\Exception $e) {
            $e->getMessage();
            throw new \Exception($e);
        }
    }

    public function agrupamentoEvolSave($paramsQuery)
    {
        try {
            $query = "INSERT INTO 
                      EVOLCONT(NRCONTATO, DTHREVOLCONT, CDTIPOCONT, IDSENEVOCONT, IDTPEVOLCONT, CDOPEREVOL, CDMOTIEVOCON, NRORG, CRM_NRORGTRAB, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, TXEVOLCONT, IDATIVO, NRSEQRESULEVOL, NRSEQEVOLCONT, NRSEQCONTATO, IDGERAFATURA, IDANEXOEVOLCONT, CDFILIAL)
                      VALUES(:NRCONTATO, :DTHREVOLCONT, :CDTIPOCONT, :IDSENEVOCONT, 'P', :CDOPEREVOL, :CDMOTIEVOCON, :NRORG, :CRM_NRORGTRAB, SYSDATE, SYSDATE, :NRORG, :CDOPEREVOL, :NRORG, :CDOPEREVOL, :TXEVOLCONT, 'S', :NRSEQRESULEVOL, :NRSEQEVOLCONT, :NRSEQCONTATO, 'NAO', 'N', :CDFILIAL)";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
        } catch (\Exception $e) {
            $e->getMessage();
            throw new \Exception($e);
        }
    }

    public function searchNrContato($paramsQuery)
    {
        try {
            $query = "SELECT 
                         NRCONTATO, 
                         NRSEQCONTATO 
                       FROM 
                         CONTATO
                        WHERE 
                          NRPARCNEGOCIO = :NRPARCNEGOCIO
                          AND NRORG = :NRORG";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);

            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp[0];
            } else {
                return array();
            }
        } catch (\Exception $e) {
            $e->getMessage();
        }
    }

    public function verifyParcGroupRel($paramsQuery)
    {
        try {
            $query = "SELECT
                         AGR.NRAGRPARC, 
                         PARC.NRPARCNEGOCIO,
                         PARC.NMPRINCIPALPARC,
                         PARC.NRINSCRICAOPARC
                       FROM 
                         CRM_AGRUPAMENTOPARCEIRO AGR
                         LEFT JOIN PARCNEGOCIO PARC
                           ON PARC.NRPARCNEGOCIO = AGR.NRPARCNEGOCIO_REL
                        WHERE 
                          AGR.NRPARCNEGOCIO = :NRPARCNEGOCIO
                          AND AGR.CRM_NRORGTRAB = :CRM_NRORGTRAB
                          AND AGR.NRORG = :NRORG";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);

            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp;
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function searchParcnegocio($paramsQuery)
    {
        try {
            $query = "SELECT
                        PARC.NRPARCNEGOCIO,
                        PARC.NMPRINCIPALPARC,
                        PARC.NRINSCRICAOPARC
                      FROM
                        PARCNEGOCIO PARC
                        LEFT JOIN CRM_STATUSPARC STAT
                          ON STAT.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                            AND STAT.NRORG = PARC.NRORG
                            AND STAT.CRM_NRORGTRAB = :CRM_NRORGTRAB
                      WHERE 
                        PARC.NRPARCNEGOCIO = :NRPARCNEGOCIO
                        AND PARC.NRORG = :NRORG
                        AND STAT.STATUSPARC <> 'CLIENTE'";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);

            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp[0];
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getRelationshipPartnerStatus($paramsQuery)
    {
        try {
            $query = "SELECT
                        NRPARCNEGOCIO,
                        NMPRINCIPALPARC,
                        NRINSCRICAOPARC
                      FROM
                        PARCNEGOCIO PARC
                      WHERE 
                        NRPARCNEGOCIO = :NRPARCNEGOCIO
                        AND AGR.NRORG = :NRORG";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp[0];
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function setRelationshipPartnerStatus($paramsQuery)
    {
        try {
            $query = "UPDATE
                        CRM_STATUSPARC
                      SET
                        STATUSPARC = :STATUSPARC
                      WHERE
                        NRORG = :NRORG
                        AND CRM_NRORGTRAB = :CRM_NRORGTRAB
                        AND NRPARCNEGOCIO = :NRPARCNEGOCIO
                     ";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findNewFilterCriteriaFactory($paramsQuery, $limit = 25, $page = 1)
    {
        try {

            if (session_status() == PHP_SESSION_NONE) {
                session_start();
            }

            if (!isset($_SESSION['current_page'])) {
                $_SESSION['current_page'] = 1;
            }

            $page = $_SESSION['current_page'];
            $offset = ($page - 1) * $limit;

            $queryBuilder  = $this->connection->createQueryBuilder();

            $queryBuilder->select('PARCEIRO.INSCRICAO, PARCEIRO.NRPARCNEGOCIO, PARCEIRO.NOMEPRINCIPAL,
            PARCEIRO.NOMEFANTASIA, PARCEIRO.IDATIVO, PARCEIRO.TIPOPARCEIRO,
            PARCEIRO.NMOPERADOR, PARCEIRO.NRPARCRESPONSAVEL, PARCEIRO.NRSEQCONTATO,
            PARCEIRO.RESULTULTEVOL, PARCEIRO.MOTIVULTEVOL, PARCEIRO.NMTIPOCONT,
            PARCEIRO.NMMOTIEVOCON, PARCEIRO.DTHREVOLCONT, PARCEIRO.DIASULTEVOL, 
            PARCEIRO.DIASULTATIVCONC, PARCEIRO.CDPAIS, PARCEIRO.NMPAIS, 
            PARCEIRO.SGESTADO, PARCEIRO.NMESTADO, PARCEIRO.CDMUNICIPIO, 
            PARCEIRO.NMMUNICIPIO, PARCEIRO.CDBAIRRO, PARCEIRO.NMBAIRRO,
            PARCEIRO.NRORG, PARCEIRO.CRM_NRORGTRAB, PARCEIRO.DESCRESULEVOL,
            PARCEIRO.CDSEGMER, PARCEIRO.DSSEGMER, PARCEIRO.POSSUITAREFAS,
            PARCEIRO.POSSUIEVOLUCOES, PARCEIRO.IDSENEVOCONT')
                ->from('(' . $this->getSubquery() . ')', 'PARCEIRO')
                ->where($queryBuilder->expr()->eq('PARCEIRO.RN', 1));

            if (isset($paramsQuery[0]['CDSEGMER']) && is_array($paramsQuery[0]['CDSEGMER'])) {
                $this->applyCdsegmerFilter($queryBuilder, $paramsQuery[0]['CDSEGMER']);
            }

            if (isset($paramsQuery[0]['INSCRICAO'])) {
                $this->applyInscriptionFilter($queryBuilder, $paramsQuery[0]['INSCRICAO']);
            }

            if (isset($paramsQuery[0]['NOMEPRINCIPAL'])) {
                $this->applyLegalNameFilter($queryBuilder, $paramsQuery[0]['NOMEPRINCIPAL']);
            }

            if (isset($paramsQuery[0]['NOMEFANTASIA'])) {
                $this->applyDoingBusinessFilter($queryBuilder, $paramsQuery[0]['NOMEFANTASIA']);
            }

            if (isset($paramsQuery[0]['IDATIVO'])) {
                $this->applyIdAtivo($queryBuilder, $paramsQuery[0]['IDATIVO']);
            }

            if (isset($paramsQuery[0]['TIPOPARCEIRO'])) {
                $this->applyPartnerTypeFilter($queryBuilder, $paramsQuery[0]['TIPOPARCEIRO']);
            }

            if (isset($paramsQuery[0]['STATUS'])) {
                $this->applyPartnerStatusFilter($queryBuilder, $paramsQuery[0]['STATUS']);
            }

            if (isset($paramsQuery[0]['NRSEQRESULEVOL']) && is_array($paramsQuery[0]['NRSEQRESULEVOL'])) {
                $this->applyResultOfLastEvolutionFilter($queryBuilder, $paramsQuery[0]['NRSEQRESULEVOL']);
            }

            if (isset($paramsQuery[0]['CDMOTIEVOCON']) && is_array($paramsQuery[0]['CDMOTIEVOCON'])) {
                $this->applyLastEvolutionReasonFilter($queryBuilder, $paramsQuery[0]['CDMOTIEVOCON']);
            }

            if (isset($paramsQuery[0]['CDPAIS'])) {
                $this->applyCountryFilter($queryBuilder, $paramsQuery[0]['CDPAIS']);
            }

            if (isset($paramsQuery[0]['SGESTADO'])) {
                $this->applyStateFilter($queryBuilder, $paramsQuery[0]['SGESTADO']);
            }

            if (isset($paramsQuery[0]['CDMUNICIPIO'])) {
                $this->applyCityFilter($queryBuilder, $paramsQuery[0]['CDMUNICIPIO']);
            }

            if (isset($paramsQuery[0]['CDBAIRRO'])) {
                $this->applyNeighborhoodFilter($queryBuilder, $paramsQuery[0]['CDBAIRRO']);
            }

            if (isset($paramsQuery[0]['POSSUIEVOLUCOES'])) {
                $this->applyhasEvolutionsFilter($queryBuilder, $paramsQuery[0]['POSSUIEVOLUCOES']);
            }

            if (isset($paramsQuery[0]['POSSUITAREFAS'])) {
                $this->applyhasTaskFilter($queryBuilder, $paramsQuery[0]['POSSUITAREFAS']);
            }

            if (isset($paramsQuery[0]['DIASULTEVOL'])) {
                $this->applyDaysWithoutEvolutionFilter($queryBuilder, $paramsQuery[0]['DIASULTEVOL']);
            }

            if (isset($paramsQuery[0]['DIASULTATIVCONC'])) {
                $this->applyNoTasksDoneFilter($queryBuilder, $paramsQuery[0]['DIASULTATIVCONC']);
            }

            if ((isset($paramsQuery[0]['DTINICIAL'])) && (isset($paramsQuery[0]['DTFIM']))) {
                $this->applyDateRangeFilter($queryBuilder, $paramsQuery[0]['DTINICIAL'], $paramsQuery[0]['DTFIM']);
            }

            if (isset($paramsQuery[0]['NRPARCRESPONSAVEL']) && is_array($paramsQuery[0]['NRPARCRESPONSAVEL'])) {
                $this->applyOperRespFilter($queryBuilder, $paramsQuery[0]['NRPARCRESPONSAVEL']);
            }

            $stmt = $queryBuilder->setFirstResult($offset)->setMaxResults($limit)->execute();
            $results = $stmt->fetchAll();
            $_SESSION['current_page']++;
            return $results;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getSubquery()
    {
        $subquery = " SELECT
        PARC.NRINSCRICAOPARC AS INSCRICAO,
        OPER.NMOPERADOR,
        STAT.NRPARCRESP AS NRPARCRESPONSAVEL,
        PARC.NRPARCNEGOCIO,
        PARC.NMPRINCIPALPARC AS NOMEPRINCIPAL,
        PARC.NMSECUNDARIPARC AS NOMEFANTASIA,
        EVOL.DTHREVOLCONT,
        EVOL.NRCONTATO,
        EVOL.NRSEQCONTATO,
        EVOL.NRSEQRESULEVOL AS RESULTULTEVOL,
        PARC.NRORG,
        PARC.CDTIPOPARCPRINCIPAL AS TIPOPARCEIRO,
        PARC.CDSEGMER,
        EVOL.CDOPEREVOL,
        EVOL.CDTIPOCONT,
        EVOL.CDMOTIEVOCON AS MOTIVULTEVOL,  
        EVOL.CRM_NRORGTRAB,
        TPCONT.NMTIPOCONT,
        MTECONT.NMMOTIEVOCON,
        RESEVOL.DESCRESULEVOL,
        EVOL.IDSENEVOCONT,
        ENDEPARC.CDPAIS,
        ENDEPARC.SGESTADO,
        ENDEPARC.CDMUNICIPIO,
        ENDEPARC.CDBAIRRO,
        PAIS.NMPAIS,
        CASE WHEN FILA.NRSEQFILACONTA IS NULL THEN 'N' ELSE 'S' END AS POSSUITAREFAS,
        CASE WHEN EVOL.DTHREVOLCONT IS NULL THEN 'N' ELSE 'S' END AS POSSUIEVOLUCOES,
        EST.NMESTADO,
        MUNI.NMMUNICIPIO,
        BAIRRO.NMBAIRRO,
        FILA.DTREALFILACONTA,
        SEG.DSSEGMER,
        STAT.IDATIVO,
        STAT.NRPARCRESP,
        TRUNC(SYSDATE - EVOL.DTHREVOLCONT) AS DIASULTEVOL,
        TRUNC(SYSDATE - FILA.DTREALFILACONTA) AS DIASULTATIVCONC,
        ROW_NUMBER() OVER (PARTITION BY PARC.NRPARCNEGOCIO ORDER BY EVOL.DTHREVOLCONT DESC) AS RN
    FROM PARCNEGOCIO PARC
    LEFT JOIN CRM_STATUSPARC STAT ON PARC.NRPARCNEGOCIO = STAT.NRPARCNEGOCIO AND STAT.NRORG = PARC.NRORG
    LEFT JOIN OPERADOR OPER ON OPER.CDOPERADOR = PARC.CDOPERRESPONSAVEL AND OPER.CDOPERADOR = STAT.CDOPERRESP AND OPER.NRORG = PARC.NRORG
    LEFT JOIN CRM_CANDIDATO LEAD ON LEAD.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO AND LEAD.NRORG = PARC.NRORG
    LEFT JOIN SEGMERCADO SEG ON SEG.CDSEGMER = PARC.CDSEGMER AND LEAD.CDSEGMER = PARC.CDSEGMER AND SEG.NRORG = PARC.NRORG
    LEFT JOIN EVOLCONT EVOL ON EVOL.NRSEQCONTATO = LEAD.NRSEQCONTATO AND EVOL.NRORG = PARC.NRORG
    LEFT JOIN FILACONTATO FILA ON FILA.NRSEQFILACONTA = EVOL.NRSEQFILACONTA AND FILA.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO AND FILA.NRSEQCONTATO = EVOL.NRSEQCONTATO AND FILA.NRORG = PARC.NRORG
    LEFT JOIN TIPOCONT TPCONT ON TPCONT.CDTIPOCONT = EVOL.CDTIPOCONT AND TPCONT.NRORG = PARC.NRORG 
    LEFT JOIN MOTIEVOLCONT MTECONT ON MTECONT.CDMOTIEVOCON = EVOL.CDMOTIEVOCON AND MTECONT.NRORG = PARC.NRORG
    LEFT JOIN CRM_RESULTEVOL RESEVOL ON RESEVOL.NRSEQRESULEVOL = EVOL.NRSEQRESULEVOL AND RESEVOL.NRORG = PARC.NRORG
    LEFT JOIN ENDERECOPARC ENDEPARC ON ENDEPARC.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO AND ENDEPARC.NRORG = PARC.NRORG
    LEFT JOIN PAIS ON PAIS.CDPAIS = ENDEPARC.CDPAIS AND PAIS.NRORG = ENDEPARC.NRORG
    LEFT JOIN ESTADO EST ON EST.CDPAIS = PAIS.CDPAIS AND EST.SGESTADO = ENDEPARC.SGESTADO AND EST.NRORG = ENDEPARC.NRORG
    LEFT JOIN MUNICIPIO MUNI ON MUNI.SGESTADO = EST.SGESTADO AND MUNI.CDMUNICIPIO = ENDEPARC.CDMUNICIPIO AND MUNI.NRORG = ENDEPARC.NRORG
    LEFT JOIN BAIRRO ON BAIRRO.CDMUNICIPIO = MUNI.CDMUNICIPIO AND BAIRRO.CDBAIRRO = ENDEPARC.CDBAIRRO AND BAIRRO.NRORG = ENDEPARC.NRORG
    WHERE PARC.CDTIPOPARCPRINCIPAL NOT IN ('PESSOA', 'OPERADOR', 'REVENDA', 'INTERLOCUTOR', 'FORNECEDOR', 'ESTRUTURA', 'FUNCIONARIO', 'CLIENTE')";

        return $subquery;
    }


    private function applyInscriptionFilter($qb, $inscription)
    {
        $qb->andWhere($qb->expr()->like('PARCEIRO.INSCRICAO', ':inscription'))
            ->setParameter('inscription', '%' . $inscription . '%');
    }

    private function applyLegalNameFilter($qb, $legalName)
    {
        $qb->andWhere($qb->expr()->like('PARCEIRO.NOMEPRINCIPAL', ':legalName'))
            ->setParameter('legalName', '%' . $legalName . '%');
    }

    private function applyDoingBusinessFilter($qb, $doingBusiness)
    {
        $qb->andWhere($qb->expr()->like('PARCEIRO.NOMEFANTASIA', ':doingBusiness'))
            ->setParameter('doingBusiness', '%' . $doingBusiness . '%');
    }

    private function applyCdsegmerFilter($qb, $cdsegmer)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.CDSEGMER', ':cdsegmer'))
            ->setParameter('cdsegmer', $cdsegmer, Connection::PARAM_STR_ARRAY);
    }

    private function applyDateRangeFilter($qb, $startDate, $endDate)
    {
        $qb->andWhere(
            $qb->expr()->andX(
                $qb->expr()->gte('PARCEIRO.DTHREVOLCONT', 'TO_DATE(:startDate, \'YYYY-MM-DD\')'),
                $qb->expr()->lte('PARCEIRO.DTHREVOLCONT', 'TO_DATE(:endDate, \'YYYY-MM-DD\')')
            )
        )
            ->setParameter('startDate', $startDate)
            ->setParameter('endDate', $endDate);
    }

    private function applyDaysWithoutEvolutionFilter($qb, $daysWithoutUpdate)
    {
        $qb->andWhere('PARCEIRO.DIASULTEVOL >= :daysWithoutUpdate')
            ->setParameter('daysWithoutUpdate', $daysWithoutUpdate);
    }

    private function applyNoTasksDoneFilter($qb, $daysWithoutTasksDone)
    {
        $qb->andWhere('PARCEIRO.DIASULTATIVCONC >= :daysWithoutTasksDone')
            ->setParameter('daysWithoutTasksDone', $daysWithoutTasksDone);
    }

    private function applyResultOfLastEvolutionFilter($qb, $resultOfLastEvolution)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.RESULTULTEVOL', ':resultOfLastEvolution'))
            ->setParameter('resultOfLastEvolution', $resultOfLastEvolution, Connection::PARAM_STR_ARRAY);
    }

    private function applyLastEvolutionReasonFilter($qb, $evolutionReason)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.MOTIVULTEVOL', ':evolutionReason'))
            ->setParameter('evolutionReason', $evolutionReason, Connection::PARAM_STR_ARRAY);
    }

    private function applyIdAtivo($qb, $idAtivo)
    {
        $qb->andWhere($qb->expr()->eq('PARCEIRO.IDATIVO', ':idAtivo'))
            ->setParameter('idAtivo', $idAtivo);
    }

    private function applyPartnerTypeFilter($qb, $partnerType)
    {
        $qb->andWhere($qb->expr()->eq('PARCEIRO.TIPOPARCEIRO', ':partnerType'))
            ->setParameter('partnerType', $partnerType);
    }

    private function applyPartnerStatusFilter($qb, $statusPartner)
    {
        $qb->andWhere($qb->expr()->eq('PARCEIRO.STATUS', ':statusPartner'))
            ->setParameter('statusPartner', $statusPartner);
    }

    private function applyCountryFilter($qb, $country)
    {
        $qb->andWhere($qb->expr()->eq('PARCEIRO.CDPAIS', ':country'))
            ->setParameter('country', $country);
    }

    private function applyStateFilter($qb, $state)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.SGESTADO', ':state'))
            ->setParameter('state', $state);
    }

    private function applyCityFilter($qb, $city)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.CDMUNICIPIO', ':city'))
            ->setParameter('city', $city);
    }

    private function applyhasEvolutionsFilter($qb, $hasEvolution)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.POSSUIEVOLUCOES', ':hasEvolution'))
            ->setParameter('hasEvolution', $hasEvolution);
    }

    private function applyhasTaskFilter($qb, $hasTask)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.POSSUITAREFAS', ':hasTask'))
            ->setParameter('hasTask', $hasTask);
    }

    private function applyNeighborhoodFilter($qb, $neighborhood)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.CDBAIRRO', ':neighborhood'))
            ->setParameter('neighborhood', $neighborhood);
    }

    private function applyOperRespFilter($qb, $parcResp)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.NRPARCRESPONSAVEL', ':parcResp'))
            ->setParameter('parcResp', $parcResp, Connection::PARAM_STR_ARRAY);
    }
}
