IF EXISTS (SELECT *
                   FROM SYSOBJECTS O, SYSUSERS U
                 WHERE O.NAME = 'CALCULA_EMB_SERV' AND
                             TYPE = 'P' AND
                             U.UID = O.UID AND
                             U.NAME = USER
                )
DROP PROCEDURE CALCULA_EMB_SERV
GO

CREATE PROCEDURE CALCULA_EMB_SERV
(
   @P_CDFILIAL VARCHAR(4),
   @P_CDSERVFILI VARCHAR(5),
   @P_DTCARDAPIO DATETIME,
   @P_CDCLIENTE VARCHAR(21),
   @P_CDDESTFILI VARCHAR(3),
   @P_SUBSTITUI_DIST_ATUAL VARCHAR(1),
   @P_RETORNO VARCHAR(200) OUTPUT
)
AS 
   DECLARE @V_CHAVE VARCHAR(100);
   /*Nmero do cardpio*/
   DECLARE @V_NRCARDAPIO VARCHAR(10);
   /*Indicador de servio transportado*/
   DECLARE @V_IDSERVTRAN VARCHAR(1);
   /*Chave da distribuio do cardpio*/
   DECLARE @V_NRDISTCARD NUMERIC;
   /*Chave da embalagem da distribuio do cardpio*/
   DECLARE @V_NREMBDISTCARD NUMERIC;
   
   /*Para indicar qual das configuraes de embalagem (servio ou ponto de entrega) ser utilizada*/
   DECLARE @V_CONF_A_UTILIZAR INT;
   
   /*Indica que a embalagem  nova e deve ser inserida na tabela DISTEMBCARD*/
   DECLARE @PRIMEIRA_ITERACAO_GRUPO BIT;
   /*Muda a cada vez que a embalagem trocar*/
   DECLARE @V_NRGRUPO_ATUAL NUMERIC;
   
   /*Indica que o prato  representante da embalagem*/
   DECLARE @V_IDPRINCIPAL VARCHAR(1);
   
   /*Sequencial da novo cdigo*/
   DECLARE @V_SEQUENCIAL VARCHAR(20);

   /*Comensais do ponto de entrega*/
   DECLARE @V_QTCOMESTICLI FLOAT;
   
   /*Nmero de comensais do cardpio do servio*/
   DECLARE @V_QTCOMESTICAR FLOAT;
   
   /*Total de comensais do dia*/
   DECLARE @V_QTTOTCOMENS FLOAT;

   /*=========================*/
   /*===== CURSOR DOS PRATOS =====*/
   /*=========================*/
   
   /*CURSOR DOS PRATOS*/   
   DECLARE @V_NRSEQPRATCAR NUMERIC(3);
   DECLARE @V_CDFILAUXPRAT VARCHAR(4);
   DECLARE @V_CDPRATO VARCHAR(10);
   DECLARE @V_VRPECONSCARD FLOAT;   
   DECLARE @V_NRGRUPO NUMERIC;
   DECLARE @V_CDCNFDISTEMBSERV VARCHAR(5);
   DECLARE @V_IDPROPOSITO VARCHAR(1);
   DECLARE @V_CDCONTAINER VARCHAR(5);
   DECLARE @V_VRQTDEPRATEMB FLOAT;
   DECLARE @V_QTPORPESSOA FLOAT;
   DECLARE @V_QTTOTEXTRA FLOAT;
   
   /*Comensais do prato*/
   DECLARE @V_COMENSAIS FLOAT;
   /*Quantidade total do prato*/
   DECLARE @V_QTPRATO FLOAT;
   
   /*Quantidade de container*/
   DECLARE @V_QTCONTAINER FLOAT;
   
   
      
BEGIN
   /*=======================================*/
   /*===== VERIFICAO DO SERVIO E DO CARDPIO =====*/
   /*=======================================*/

   SELECT @V_NRCARDAPIO = C.NRCARDAPIO, @V_IDSERVTRAN = S.IDSERVTRAN, @V_NRDISTCARD  = D.NRDISTCARD,
          @V_QTCOMESTICLI = ISNULL(CL.QTCOMESTICLI, 0), @V_QTCOMESTICAR = ISNULL(C.QTCOMESTICAR, 0)
   FROM CARDADIA C
      INNER JOIN SERVICO S ON S.CDFILIAL = C.CDFILIAL AND
                              S.CDSERVFILI = C.CDSERVFILI
      LEFT JOIN CARDACLI CL ON CL.CDFILIAL = C.CDFILIAL AND
                                CL.NRCARDAPIO = C.NRCARDAPIO AND
                                CL.CDCLIENTE = @P_CDCLIENTE AND
                                CL.CDDESTFILI = @P_CDDESTFILI
      LEFT JOIN DISTCARD D ON D.CDFILIAL = C.CDFILIAL AND
                              D.NRCARDAPIO = C.NRCARDAPIO AND
                              ((@P_CDCLIENTE IS NULL) OR 
                               (D.CDCLIENTE = @P_CDCLIENTE AND D.CDDESTFILI = @P_CDDESTFILI))
   WHERE C.DTCARDAPIO = @P_DTCARDAPIO
     AND C.CDFILIAL = @P_CDFILIAL
     AND C.CDSERVFILI = @P_CDSERVFILI;
   
   IF @V_NRCARDAPIO IS NULL BEGIN
      SET @P_RETORNO = 'ERRO_N_EXIST_CARD';
      RETURN;
   END;
   
   IF @V_IDSERVTRAN IS NOT NULL BEGIN
      IF @V_IDSERVTRAN <> 'S' BEGIN
         SET @P_RETORNO = 'ERRO_SERV_N_TRANS';
         RETURN;
      END;
   END;
   
   IF @P_CDCLIENTE IS NOT NULL AND (@V_QTCOMESTICLI <= 0) BEGIN
      SET @P_RETORNO = 'ERRO_COMESTICLI_ZERO';
      RETURN;
   END
   ELSE IF (@V_QTCOMESTICAR <= 0) BEGIN
      SET @P_RETORNO = 'ERRO_COMESTICAR_ZERO';
      RETURN;
   END;
   
   IF @P_CDCLIENTE IS NOT NULL BEGIN
      SET @V_QTTOTCOMENS = @V_QTCOMESTICLI;
   END
   ELSE BEGIN
      SET @V_QTTOTCOMENS = @V_QTCOMESTICAR;
   END;
   
   
   
   /*============================*/
   /*===== DISTRIBUICAO EXISTENTE =====*/
   /*============================*/   
   
   IF @V_NRDISTCARD IS NOT NULL BEGIN
      IF @P_SUBSTITUI_DIST_ATUAL = 'S' BEGIN
         /*Apaga a distribuio existente para o cardpio*/
         DELETE FROM ITEMBDISTCARD
         WHERE NRDISTCARD = @V_NRDISTCARD;
         
         DELETE FROM EMBDISTCARD
         WHERE NRDISTCARD = @V_NRDISTCARD;
         
         DELETE FROM CNFDISTCARD
         WHERE NRDISTCARD = @V_NRDISTCARD;
         
         DELETE FROM DISTCARD
         WHERE NRDISTCARD = @V_NRDISTCARD;
      END
      ELSE BEGIN
         /*Consiste se houve alterao do cardpio que invalide a distribuio : */
         /*Mudana de prato, excluso de uma coluna ou incluso de uma coluna na elaborao*/
         
         SELECT @V_NRSEQPRATCAR = ISNULL(C.NRSEQPRATCAR, CNF.NRSEQPRATCAR)
           FROM COMPCARD C
               FULL OUTER JOIN CNFDISTCARD CNF
                  ON CNF.NRSEQPRATCAR = C.NRSEQPRATCAR AND
                     CNF.NRDISTCARD = @V_NRDISTCARD
          WHERE C.CDFILIAL = @P_CDFILIAL
            AND C.NRCARDAPIO = @V_NRCARDAPIO
            AND ((C.CDPRATO <> CNF.CDPRATO) OR (C.CDFILAUXPRAT <> CNF.CDFILAUXPRAT) OR (CNF.NRDISTCARD IS NULL) OR (C.NRSEQPRATCAR IS NULL));
         
         
         /*Indica que houve mudana nos pratos do cardpio*/
         IF @V_NRSEQPRATCAR IS NOT NULL BEGIN
            SET @P_RETORNO = 'ERRO_PRAT_CARD_ALT';
            RETURN;
         END
         ELSE BEGIN
            SET @P_RETORNO = CONVERT(VARCHAR, @V_NRDISTCARD);
            RETURN;
         END;
      END;
   END;

   /*=============================================================*/
   /*DETERMINA QUAL CONFIGURAO UTILIZAR : SE A DO SERVIO OU A DO PONTO DE ENTREGA*/
   /*=============================================================*/
   
   SELECT @V_CONF_A_UTILIZAR = MAX(CONF_A_UTILIZAR)         
   FROM
   (
     /*VERIFICA SE H CONFIGURAO DE EMBALAGENS PARA O SERVIO*/
     SELECT 1 AS CONF_A_UTILIZAR     
     WHERE EXISTS
     (
       SELECT 1
       FROM CNFDISTEMBSERV
       WHERE CDFILIAL = @P_CDFILIAL 
       AND CDSERVFILI = @P_CDSERVFILI
       AND CDCLIENTE IS NULL 
     )
     
     UNION
     /*VERIFICA SE H CONFIGURAO DE EMBALAGENS PARA O PONTO DE ENTREGA*/
     SELECT 2 AS CONF_A_UTILIZAR     
     WHERE EXISTS
     (
       SELECT 1
       FROM CNFDISTEMBSERV
       WHERE CDFILIAL = @P_CDFILIAL 
         AND CDSERVFILI = @P_CDSERVFILI
         AND CDCLIENTE = @P_CDCLIENTE
         AND CDDESTFILI = @P_CDDESTFILI
     )
   ) S;
         
   IF @V_CONF_A_UTILIZAR IS NULL BEGIN
      /*Indica que no h configurao a ser utilizada*/
      SET @V_CONF_A_UTILIZAR = 0;
   END;
   
   /*Inclui distribuio do cardapio*/
   /*Busca novo sequencial*/               
   SET @V_CHAVE = 'DISTCARD';
   EXEC NOVO_CODIGO_SC @V_CHAVE, @V_SEQUENCIAL OUTPUT;
   SET @V_NRDISTCARD = CONVERT(NUMERIC, @V_SEQUENCIAL);
      
   INSERT INTO DISTCARD
   (NRDISTCARD, CDFILIAL,NRCARDAPIO, CDCLIENTE, CDDESTFILI, QTCOMENSAIS)
   VALUES
   (@V_NRDISTCARD, @P_CDFILIAL, @V_NRCARDAPIO, @P_CDCLIENTE, @P_CDDESTFILI, @V_QTTOTCOMENS);


   /*USADA PARA INDICAR QUEBRA DE PROCESSAMENTO DE EMBALAGENS*/
   SET @V_NRGRUPO_ATUAL = -1; 
   
   
   /*=========================*/
   /*===== CURSOR DOS PRATOS =====*/
   /*=========================*/
   DECLARE C_PRATOS CURSOR LOCAL
   FOR
      SELECT C.NRSEQPRATCAR, C.CDFILAUXPRAT, C.CDPRATO, C.VRPECONSCARD,
           CNF.NRGRUPO, CNF.CDCNFDISTEMBSERV, CNF.IDPROPOSITO, CNF.CDCONTAINER,
           R.VRQTDEPRATEMB, ISNULL(QTPORPESSOA, 0) QTPORPESSOA, ISNULL(QTTOTEXTRA, 0) QTTOTEXTRA
      FROM COMPCARD C
           /*Configurao de embalagem para o tipo de prato*/
           LEFT JOIN 
           (
           SELECT CNF.NRSEQCNFCARD, CNF.NRGRUPO, CNF.CDCNFDISTEMBSERV, CNF.CDCONTAINER, C.IDPROPOSITO
           FROM CNFDISTEMBSERV CNF
                LEFT JOIN CONTAINER C ON C.CDCONTAINER = CNF.CDCONTAINER
           WHERE CDFILIAL = @P_CDFILIAL
             AND CDSERVFILI = @P_CDSERVFILI
             AND CDCLIENTE IS NULL
             AND @V_CONF_A_UTILIZAR = 1
           UNION
           SELECT CNF.NRSEQCNFCARD, CNF.NRGRUPO, CNF.CDCNFDISTEMBSERV, CNF.CDCONTAINER, C.IDPROPOSITO
           FROM CNFDISTEMBSERV CNF
                LEFT JOIN CONTAINER C ON C.CDCONTAINER = CNF.CDCONTAINER
           WHERE CDFILIAL = @P_CDFILIAL
             AND CDSERVFILI = @P_CDSERVFILI
             AND CDCLIENTE = @P_CDCLIENTE
             AND CDDESTFILI = @P_CDDESTFILI
             AND @V_CONF_A_UTILIZAR = 2
          ) CNF ON CNF.NRSEQCNFCARD = C.NRSEQPRATCAR
          /*Relao entre embalagem selecionada e prato*/
          LEFT JOIN RELPRATEMB R ON R.CDFILAUXPRAT = C.CDFILAUXPRAT AND
                                    R.CDPRATO = C.CDPRATO AND
                                    R.CDCONTAINER = CNF.CDCONTAINER
         /*Peso unitrio e quantidade de amostra (extra) do prato*/
          LEFT JOIN
          (
            SELECT CDFILAUXPRAT, CDPRATO, QTPADRAOPRAT AS QTPORPESSOA, QTAMOSTRPADR AS QTTOTEXTRA, 'P' AS RECE_UTIL
            FROM PRATPADR            

            UNION 

            SELECT CDFILAUXPRAT, CDPRATO, QTPADRAOFILI AS QTPORPESSOA, QTAMOSTRFILI AS QTTOTEXTRA, 'F' AS RECE_UTIL
            FROM PRATFILI
            WHERE CDFILIAL = @P_CDFILIAL
            
            UNION 

            SELECT CDFILAUXPRAT, CDPRATO, QTPADRAOSERV AS QTPORPESSOA, QTAMOSTRSERV AS QTTOTEXTRA, 'S' AS RECE_UTIL
            FROM PRATSERV
            WHERE CDFILIAL = @P_CDFILIAL AND
                        CDSERVFILI = @P_CDSERVFILI
            
          ) P ON P.CDFILAUXPRAT = C.CDFILAUXPRAT AND
                    P.CDPRATO = C.CDPRATO AND
                    teknisa.RECEITA_UTILIZADA(C.CDPRATO, C.CDFILAUXPRAT, @P_CDFILIAL, @P_CDSERVFILI) = P.RECE_UTIL
                    
      WHERE C.CDFILIAL = @P_CDFILIAL AND
            C.NRCARDAPIO = @V_NRCARDAPIO
      ORDER BY NRGRUPO;
   
   OPEN C_PRATOS;
   
   FETCH NEXT FROM C_PRATOS
    INTO @V_NRSEQPRATCAR, @V_CDFILAUXPRAT, @V_CDPRATO, @V_VRPECONSCARD,            
               @V_NRGRUPO, @V_CDCNFDISTEMBSERV, @V_IDPROPOSITO, @V_CDCONTAINER,                   
               @V_VRQTDEPRATEMB, @V_QTPORPESSOA, @V_QTTOTEXTRA;
   
   /*=======================================*/
   /*PERCORRE OS PRATOS DO SERVIO PARA O DIA INFORMADO*/      
   /*=======================================*/
   
   WHILE (@@FETCH_STATUS = 0) BEGIN
   
      SET @V_COMENSAIS = ROUND((@V_VRPECONSCARD/100) * @V_QTTOTCOMENS, 5);         
      SET @V_QTPRATO = ROUND(@V_COMENSAIS * @V_QTPORPESSOA, 5);
      
      /*Insere a configurao do cardpio considerada na distribuio, bem como outras grandezas envolvidas*/
      INSERT INTO CNFDISTCARD
      (NRDISTCARD, NRSEQPRATCAR, CDFILAUXPRAT, CDPRATO, QTTOTCOMENSAIS, 
       QTTOTPRATO, QTPORPESSOA, QTTOTEXTRA, IDCADAUTO)
      VALUES
      (@V_NRDISTCARD, @V_NRSEQPRATCAR, @V_CDFILAUXPRAT, @V_CDPRATO, @V_COMENSAIS, 
       @V_QTPRATO, @V_QTPORPESSOA, @V_QTTOTEXTRA, 'S');
      
      /*VERIFICA QUEBRA DE GRUPO (NOVA EMBALAGEM SENDO PROCESSADA)*/
      IF @V_NRGRUPO_ATUAL <> @V_NRGRUPO BEGIN
         SET @V_NRGRUPO_ATUAL = @V_NRGRUPO;
         /*INDICA MOMENTO DE INSERO DAS EMBALAGENS*/
         SET @PRIMEIRA_ITERACAO_GRUPO = 1;
      END;
      
      /*===========================*/
      /*===== EMBALAGENS COLETIVAS =====*/
      /*===========================*/
      /*Se a embalagem e o prato no foram relacionados, no faz a distribuio deste na embalagem*/
      IF (@V_IDPROPOSITO = 'C') AND (@V_VRQTDEPRATEMB IS NOT NULL) BEGIN
         IF @V_QTPORPESSOA > 0 BEGIN
            IF @PRIMEIRA_ITERACAO_GRUPO = 1 BEGIN
               /*Quantidade de containers : completos + incompleto*/
               SET @V_QTCONTAINER = ROUND(@V_QTPRATO/@V_VRQTDEPRATEMB, 0, 1);
               IF CONVERT(NUMERIC, @V_QTPRATO) % CONVERT(NUMERIC, @V_VRQTDEPRATEMB) <> 0 BEGIN
                  SET @V_QTCONTAINER = @V_QTCONTAINER + 1;
               END;
               
               /*Busca novo sequencial*/               
               SET @V_CHAVE = 'EMBDISTCARD';
               EXEC NOVO_CODIGO_SC @V_CHAVE,@V_SEQUENCIAL OUTPUT;
               SET @V_NREMBDISTCARD = CONVERT(NUMERIC, @V_SEQUENCIAL);
               
               /*Insere embalagem na distribuio do cardpio*/
               INSERT INTO EMBDISTCARD
               (NRDISTCARD, NREMBDISTCARD, CDCONTAINER, QTCONTAINER, IDCADAUTO)
               VALUES
               (@V_NRDISTCARD, @V_NREMBDISTCARD, @V_CDCONTAINER, @V_QTCONTAINER, 'S');
               
            END;
            
            /*Insere o prato na embalagem na distribuio*/
            INSERT INTO ITEMBDISTCARD
            (NRDISTCARD, NRSEQPRATCAR, NREMBDISTCARD, 
             QTCOMENSAIS, QTPRATO, QTEXTRA, IDCADAUTO, IDPRINCIPAL)
            VALUES
            (@V_NRDISTCARD, @V_NRSEQPRATCAR, @V_NREMBDISTCARD, 
             @V_COMENSAIS, @V_QTPRATO, @V_QTTOTEXTRA, 'S', 'S');
         END;
         
      /*============================*/
      /*===== EMBALAGENS INDIVIDUAIS =====*/
      /*============================*/
      END
      ELSE IF (@V_IDPROPOSITO = 'I') BEGIN
         IF @PRIMEIRA_ITERACAO_GRUPO = 1 BEGIN
            SET @V_IDPRINCIPAL = 'S';
            /*Busca novo sequencial*/
            SET @V_CHAVE = 'EMBDISTCARD';
            EXEC NOVO_CODIGO_SC @V_CHAVE,@V_SEQUENCIAL OUTPUT;
            SET @V_NREMBDISTCARD = CONVERT(NUMERIC, @V_SEQUENCIAL);
            
            /*Insere embalagem na distribuio do cardpio*/
            INSERT INTO EMBDISTCARD
            (NRDISTCARD, NREMBDISTCARD, CDCONTAINER, QTCONTAINER, IDCADAUTO)
            VALUES
            (@V_NRDISTCARD, @V_NREMBDISTCARD, @V_CDCONTAINER, @V_QTTOTCOMENS, 'S');
         END;
         
         /*Insere o prato na embalagem na distribuio*/
         INSERT INTO ITEMBDISTCARD
         (NRDISTCARD, NRSEQPRATCAR, NREMBDISTCARD, 
          QTCOMENSAIS, QTPRATO, QTEXTRA, IDCADAUTO, IDPRINCIPAL)
         VALUES
         (@V_NRDISTCARD, @V_NRSEQPRATCAR, @V_NREMBDISTCARD, 
          @V_COMENSAIS, @V_QTPRATO, @V_QTTOTEXTRA, 'S', @V_IDPRINCIPAL);
         SET @V_IDPRINCIPAL = 'N';
      END
      /*==============================================*/
      /*===== DISTRIBUIO AUTOMTICA ======================*/
      /*Para todos os que no foram contemplados pelos passos acima :          */         
      /*- Pratos  que no possuem relao explicitada com a embalagem           */
      /*- Pratos cujo tipo de prato no pertence  conf. original do servio         */
      /*- Pratos cuja distribuio foi definida como automtica na tela de servio*/
      /*==============================================*/
      
      ELSE BEGIN         
         IF @V_CDCNFDISTEMBSERV = 'DISTR' BEGIN               
            EXEC DIST_AUTO_PRAT @V_NRDISTCARD, @V_NRSEQPRATCAR,'DISTR';
         END
         ELSE IF @V_CDCNFDISTEMBSERV = 'DIST' BEGIN
            EXEC DIST_AUTO_PRAT @V_NRDISTCARD, @V_NRSEQPRATCAR, 'DIST';
         END;
      END;
      
      /*Somente  verdadeiro para a primeira iterao de uma embalagem da configurao*/
      SET @PRIMEIRA_ITERACAO_GRUPO = 0;
      
      FETCH NEXT FROM C_PRATOS
        INTO @V_NRSEQPRATCAR, @V_CDFILAUXPRAT, @V_CDPRATO, @V_VRPECONSCARD,            
               @V_NRGRUPO, @V_CDCNFDISTEMBSERV, @V_IDPROPOSITO, @V_CDCONTAINER,                   
               @V_VRQTDEPRATEMB, @V_QTPORPESSOA, @V_QTTOTEXTRA;
   
   END;
   
   CLOSE C_PRATOS;
   
   /*ATUALIZA SITUAO DA DISTRIBUIO QUANTO  SUA COMPLETUDE*/
   
   SELECT @V_NRSEQPRATCAR = MAX(C.NRSEQPRATCAR)          
   FROM CNFDISTCARD C
        LEFT JOIN
       (
        SELECT NRDISTCARD, NRSEQPRATCAR, SUM(QTCOMENSAIS) AS QTCOMENSAIS,
               SUM(QTPRATO) AS QTPRATO, SUM(QTEXTRA) AS QTEXTRA
          FROM ITEMBDISTCARD
          WHERE NRDISTCARD = @V_NRDISTCARD
        GROUP BY NRDISTCARD, NRSEQPRATCAR
       ) U ON U.NRSEQPRATCAR = C.NRSEQPRATCAR
   WHERE C.NRDISTCARD = @V_NRDISTCARD AND
         (C.QTTOTCOMENSAIS - ISNULL(U.QTCOMENSAIS, 0) > 0 OR
          C.QTTOTPRATO - ISNULL(U.QTPRATO, 0) > 0 OR
          C.QTTOTEXTRA - ISNULL(U.QTEXTRA, 0) > 0 );
   
   
   IF @V_NRSEQPRATCAR IS NOT NULL BEGIN
      UPDATE DISTCARD
         SET IDSITUACAO = 'I'
         WHERE NRDISTCARD = @V_NRDISTCARD;
   END
   ELSE BEGIN
      UPDATE DISTCARD
         SET IDSITUACAO = 'C'
         WHERE NRDISTCARD = @V_NRDISTCARD;   
   END;
   
   
   SET @P_RETORNO = CONVERT(VARCHAR, @V_NRDISTCARD);
   
    
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'CALCULA_EMB_SERV','5.75.000';
GO